@@ -1,5 +1,5 @@
 /*
- * $Id: directory.h,v 1.2.2.1 2001/12/03 05:01:04 jmarcus Exp $
+ * $Id: directory.h,v 1.13.2.4.2.2 2004/05/10 18:40:32 didg Exp $
  *
  * Copyright (c) 1990,1991 Regents of The University of Michigan.
  * All Rights Reserved.
@@ -30,6 +30,7 @@
 #include <sys/types.h>
 /*#include <sys/stat.h>*/ /* including it here causes some confusion */
 #include <netatalk/endian.h>
+#include <dirent.h>
 
 /* sys/types.h usually snarfs in major/minor macros. if they don't
  * try this file. */
@@ -50,9 +51,38 @@ struct dir {
     void        *d_ofork;            /* oforks using this directory. */
     u_int32_t   d_did;
     int	        d_flags;
-    char	*d_name;
+
+    time_t      ctime;                /* inode ctime */
+    u_int32_t   offcnt;               /* offspring count */
+
+    char	*d_m_name;            /* mac name */
+    char        *d_u_name;            /* unix name */
 };
 
+struct path {
+    int         m_type;             /* mac name type (long name, unicode */
+    char	*m_name;            /* mac name */
+    char        *u_name;            /* unix name */
+    struct dir  *d_dir;             /* */
+    int         st_valid;           /* does st_errno and st set */
+    int         st_errno;
+    struct stat st;
+};
+
+#ifndef ATACC
+static __inline__ int path_isadir(struct path *o_path)
+{
+    return o_path->d_dir != NULL;
+#if 0
+    return o_path->m_name == '\0' || /* we are in a it */
+           !o_path->st_valid ||      /* in cache but we can't chdir in it */ 
+           (!o_path->st_errno && S_ISDIR(o_path->st.st_mode)); /* not in cache an can't chdir */
+#endif
+}
+#else
+extern int path_isadir(struct path *o_path);
+#endif
+
 /* child addition/removal macros */
 #define dirchildadd(a, b) do { \
 	if (!(a)->d_child) \
@@ -78,7 +108,11 @@ struct dir {
 
 /* setgid directories */
 #ifndef DIRBITS
-#define DIRBITS S_ISGID
+# ifdef AFS
+#  define DIRBITS 0
+# else /* AFS */
+#  define DIRBITS S_ISGID
+# endif /* AFS */
 #endif /* DIRBITS */
 
 #define DIRF_FSMASK	(3<<0)
@@ -103,6 +137,7 @@ struct dir {
 #define DIRPBIT_GID	11
 #define DIRPBIT_ACCESS	12
 #define DIRPBIT_PDINFO  13         /* ProDOS Info */
+#define DIRPBIT_UNIXPR  15
 
 /* directory attribute bits (see file.h for other bits) */
 #define ATTRBIT_EXPFOLDER   (1 << 1) /* shared point */
@@ -119,6 +154,8 @@ struct dir {
 /* file/directory ids. what a mess. we scramble things in a vain attempt
  * to get something meaningful */
 #ifndef AFS
+
+#if 0
 #define CNID_XOR(a)  (((a) >> 16) ^ (a))
 #define CNID_DEV(a)   ((((CNID_XOR(major((a)->st_dev)) & 0xf) << 3) | \
 	(CNID_XOR(minor((a)->st_dev)) & 0x7)) << 24)
@@ -126,6 +163,10 @@ struct dir {
 				       & 0x00ffffff)
 #define CNID_FILE(a)  (((a) & 0x1) << 31)
 #define CNID(a,b)     (CNID_DEV(a) | CNID_INODE(a) | CNID_FILE(b))
+#endif
+
+#define CNID(a,b)     ((a)->st_ino & 0xffffffff)
+
 #else /* AFS */
 #define CNID(a,b)     (((a)->st_ino & 0x7fffffff) | CNID_FILE(b))
 #endif /* AFS */
@@ -143,24 +184,41 @@ struct maccess {
 #define	AR_UWRITE	(1<<2)
 #define	AR_UOWN		(1<<7)
 
-extern struct dir       *dirnew __P((const int));
+extern struct dir       *dirnew __P((const char *, const char *));
+extern void             dirfreename __P((struct dir *));
 extern void             dirfree __P((struct dir *));
 extern struct dir	*dirsearch __P((const struct vol *, u_int32_t));
-extern struct dir	*adddir __P((struct vol *, struct dir *, char *,
-                                               int, char *, int, struct stat *));
+extern struct dir	*dirlookup __P((const struct vol *, u_int32_t));
+extern struct dir       *dirsearch_byname __P((struct dir *,const char *));
+
+extern struct dir	*adddir __P((struct vol *, struct dir *, 
+                                               struct path *));
+
 extern struct dir       *dirinsert __P((struct vol *, struct dir *));
 extern int              movecwd __P((const struct vol *, struct dir *));
 extern int              deletecurdir __P((const struct vol *, char *, int));
-extern char		*cname __P((const struct vol *, struct dir *,
+extern struct path      *cname __P((const struct vol *, struct dir *,
                              char **));
 extern mode_t           mtoumode __P((struct maccess *));
 extern void             utommode __P((struct stat *, struct maccess *));
-extern int getdirparams __P((const struct vol *, u_int16_t, char *,
-                                 struct dir *, struct stat *, char *, int *));
-extern int setdirparams __P((const struct vol *, char *, u_int16_t, char *));
-extern int renamedir __P((char *, char *, struct dir *,
-                              struct dir *, char *, const int));
+extern int getdirparams __P((const struct vol *, u_int16_t, struct path *,
+                                 struct dir *, char *, int *));
+extern int setdirparams __P((const struct vol *, struct path *, u_int16_t, char *));
+extern int renamedir __P((const struct vol *, char *, char *, struct dir *,
+                              struct dir *, char *));
+extern int path_error __P((struct path *, int error));
 
+typedef int (*dir_loop)(struct dirent *, char *, void *);
+
+extern int  for_each_dirent __P((const struct vol *, char *, dir_loop , void *));
+
+extern int  check_access __P((char *name , int mode));
+extern int file_access   __P((struct path *path, int mode));
+
+extern int netatalk_unlink __P((const char *name));
+
+/* from enumerate.c */
+extern char *check_dirent __P((const struct vol *, char *));
 
 /* FP functions */
 extern int	afp_createdir __P((AFPObj *, char *, int, char *, int *));
@@ -171,7 +229,12 @@ extern int	afp_mapid __P((AFPObj *, char *, int, char *, int *));
 extern int	afp_mapname __P((AFPObj *, char *, int, char *, int *));
 
 /* from enumerate.c */
-extern int	afp_enumerate __P((AFPObj *, char *, int, char *, int *));
+extern int	afp_enumerate __P((AFPObj *, char *, unsigned int, char *, unsigned int *));
+extern int	afp_enumerate_ext __P((AFPObj *, char *, unsigned int, char *, unsigned int *));
+extern int	afp_enumerate_ext2 __P((AFPObj *, char *, unsigned int, char *, unsigned int *));
+
+/* from catsearch.c */
 extern int	afp_catsearch __P((AFPObj *, char *, int, char *, int *));
+extern int	afp_catsearch_ext __P((AFPObj *, char *, int, char *, int *));
 
 #endif
