@@ -1,5 +1,5 @@
 /*
- * $Id: aep.c,v 1.4.2.2 2002/03/12 15:31:12 srittau Exp $
+ * $Id: aep.c,v 1.8 2002/09/29 23:24:47 sibaz Exp $
  *
  * Copyright (c) 1990,1993 Regents of The University of Michigan.
  * All Rights Reserved. See COPYRIGHT.
@@ -11,8 +11,8 @@
 
 #include <string.h>
 #include <errno.h>
-#include <syslog.h>
 
+#include <atalk/logger.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netatalk/at.h>
@@ -32,14 +32,14 @@ int aep_packet( ap, from, data, len )
     end = data + len;
     if ( data + 2 > end || *data != DDPTYPE_AEP ||
 	    *( data + 1 ) != AEPOP_REQUEST ) {
-	syslog( LOG_INFO, "aep_packet malformed packet" );
+	LOG(log_info, logtype_atalkd, "aep_packet malformed packet" );
 	return 1;
     }
 
     *( data + 1 ) = AEPOP_REPLY;
     if ( sendto( ap->ap_fd, data, len, 0, (struct sockaddr *)from,
 	    sizeof( struct sockaddr_at )) < 0 ) {
-	syslog( LOG_ERR, "aep sendto: %s", strerror(errno) );
+	LOG(log_error, logtype_atalkd, "aep sendto: %s", strerror(errno) );
 	return 1;
     }
 
