@@ -21,18 +21,25 @@
  *	netatalk@itd.umich.edu
  */
 
+#ifndef _ATALK_RTMP_H
+#define _ATALK_RTMP_H 1
+
+#include <netatalk/endian.h>
+
 #define RTMPROP_REQUEST	1
 
 struct rtmpent {
-    u_short		re_net;
-    u_char		re_hops;
+    u_int16_t   re_net;
+    u_int8_t    re_hops;
 };
 
 #define RTMPHOPS_MAX	15
 #define RTMPHOPS_POISON	31
 
 struct rtmprdhdr {
-    u_short		rrdh_snet;
-    u_char		rrdh_idlen;
-    u_char		rrdh_id;
+    u_int16_t   rrdh_snet;
+    u_int8_t    rrdh_idlen;
+    u_int8_t    rrdh_id;
 };
+
+#endif
