@@ -1,5 +1,5 @@
 /*
- * $Id: server_child.c,v 1.7.2.2 2003/11/14 14:25:41 didg Exp $
+ * $Id: server_child.c,v 1.7.4.1.2.4.2.1 2005/03/31 00:25:55 didg Exp $
  *
  * Copyright (c) 1997 Adrian Sun (asun@zoology.washington.edu)
  * All rights reserved. See COPYRIGHT.
@@ -45,6 +45,9 @@
 #ifndef WIFSIGNALED
 #define WIFSIGNALED(status) (!WIFSTOPPED(status) && !WIFEXITED(status)) 
 #endif
+#ifndef WTERMSIG
+#define WTERMSIG(status)      ((status) & 0x7f)
+#endif
 
 #include <atalk/server_child.h>
 
@@ -57,7 +60,14 @@
 #define HASH(i) ((((i) >> 8) ^ (i)) & (CHILD_HASHSIZE - 1))
 
 struct server_child_data {
-  pid_t pid; 
+  pid_t     pid; 		/* afpd worker process pid (from the worker afpd process )*/
+  uid_t     uid;		/* user id of connected client (from the worker afpd process) */
+  int       valid;		/* 1 if we have a clientid */
+  u_int32_t time;		/* client boot time (from the mac client) */
+  int       killed;		/* 1 if we already tried to kill the client */
+
+  u_int32_t idlen;		/* clientid len (from the Mac client) */
+  char *clientid;		/* clientid (from the Mac client) */
   struct server_child_data **prevp, *next;
 };
 
@@ -159,6 +169,8 @@ int server_child_add(server_child *children, const int forkid,
   }
 
   child->pid = pid;
+  child->valid = 0;
+  child->killed = 0;
   hash_child(fork->table, child);
   children->count++;
   sigprocmask(SIG_SETMASK, &oldsig, NULL);
@@ -178,6 +190,9 @@ int server_child_remove(server_child *children, const int forkid,
     return 0;
   
   unhash_child(child);
+  if (child->clientid) {
+      free(child->clientid);
+  }
   free(child);
   children->count--;
   return 1;
@@ -197,6 +212,9 @@ void server_child_free(server_child *children)
       child = fork->table[j]; /* start at the beginning */
       while (child) {
 	tmp = child->next;
+        if (child->clientid) {
+            free(child->clientid);
+        }
 	free(child);
 	child = tmp;
       }
@@ -226,6 +244,114 @@ void server_child_kill(server_child *children, const int forkid,
   }
 }
 
+/* send kill to a child processes.
+ * a plain-old linked list 
+ * FIXME use resolve_child ?
+ */
+static int kill_child(struct server_child_data *child)
+{
+  if (!child->killed) {
+     kill(child->pid, SIGTERM);
+     /* we don't wait because there's no guarantee that we can really kill it */
+     child->killed = 1;
+     return 1;
+  }
+  else {
+     LOG(log_info, logtype_default, "Already tried to kill (%d) before! Still there?",  child->pid);
+  }
+  return 0;
+}
+
+/* -------------------- */
+void server_child_kill_one(server_child *children, const int forkid, const pid_t pid, const uid_t uid)
+{
+  server_child_fork *fork;
+  struct server_child_data *child, *tmp;
+  int i;
+
+  fork = (server_child_fork *) children->fork + forkid;
+  for (i = 0; i < CHILD_HASHSIZE; i++) {
+    child = fork->table[i];
+    while (child) {
+      tmp = child->next;
+      if (child->pid == pid) {
+          if (!child->valid) {
+             /* hmm, client 'guess' the pid, rogue? */
+             LOG(log_info, logtype_default, "Disconnecting old session (%d) no token, bailout!",  child->pid);
+          }
+          else if (child->uid != uid) {
+             LOG(log_info, logtype_default, "Disconnecting old session (%d) not the same user, bailout!",  child->pid);
+          }
+          else {
+              kill_child(child);
+          }
+      }
+      child = tmp;
+    }
+  }
+}
+
+
+/* see if there is a process for the same mac     */
+/* if the times don't match mac has been rebooted */
+void server_child_kill_one_by_id(server_child *children, const int forkid, const pid_t pid,
+          const uid_t uid, 
+          const u_int32_t idlen, char *id, u_int32_t boottime)
+{
+  server_child_fork *fork;
+  struct server_child_data *child, *tmp;
+  int i;
+
+  fork = (server_child_fork *) children->fork + forkid;
+  for (i = 0; i < CHILD_HASHSIZE; i++) {
+    child = fork->table[i];
+    while (child) {
+      tmp = child->next;
+      if ( child->pid != pid) {
+          if ( child->idlen == idlen && !memcmp(child->clientid, id, idlen)) {
+	     if ( child->time != boottime ) {
+	          if (uid == child->uid) {
+	              if (kill_child(child)) {
+		          LOG(log_info, logtype_default, "Disconnecting old session %d, client rebooted.",  child->pid);
+		      }
+		  }
+		  else {
+		      LOG(log_info, logtype_default, "Disconnecting old session not the same uid, bailout!");
+		  }
+	     }
+	     else if (child->killed) {
+	          /* there's case where a Mac close a connection and restart a new one before
+	           * the first is 'waited' by the master afpd process
+	          */
+		  LOG(log_info, logtype_default, 
+		      "WARNING: connection (%d) killed but still there.", child->pid);
+	     }
+	     else {
+		  LOG(log_info, logtype_default, 
+		      "WARNING: 2 connections (%d, %d), boottime identical, don't know if one needs to be disconnected.",
+		       child->pid, pid);
+	     } 
+		
+	  }
+      }
+      else 
+      {
+	  child->time = boottime;
+	  /* free old token if any */
+	  if (child->clientid) {
+	      free(child->clientid);
+	  }
+          child->uid = uid; 
+          child->valid = 1;
+	  child->idlen = idlen;
+          child->clientid = id;
+	  LOG(log_info, logtype_default, "Setting clientid (len %d) for %d, boottime %X", idlen, child->pid, boottime);
+      }
+      child = tmp;
+    }
+  }
+}
+
 /* for extra cleanup if necessary */
 void server_child_setup(server_child *children, const int forkid,
 			  void (*fcn)(const pid_t))
@@ -240,7 +366,6 @@ void server_child_setup(server_child *children, const int forkid,
 /* keep track of children. */
 void server_child_handler(server_child *children)
 {
-  server_child_fork *fork;
   int status, i;
   pid_t pid;
   
@@ -251,12 +376,15 @@ void server_child_handler(server_child *children)
   while ((pid = waitpid(WAIT_ANY, &status, WNOHANG)) > 0) {
     for (i = 0; i < children->nforks; i++) {
       if (server_child_remove(children, i, pid)) {
+        server_child_fork *fork;
+        
 	fork = (server_child_fork *) children->fork + i;
 	if (fork->cleanup)
 	  fork->cleanup(pid);
 	break;
       }
     }
+
     if (WIFEXITED(status)) {
       if (WEXITSTATUS(status)) {
 	LOG(log_info, logtype_default, "server_child[%d] %d exited %d", i, pid, 
@@ -267,7 +395,8 @@ void server_child_handler(server_child *children)
     } else {
       if (WIFSIGNALED(status))
       { 
-	LOG(log_info, logtype_default, "server_child[%d] %d killed", i, pid);
+	LOG(log_info, logtype_default, "server_child[%d] %d killed by signal %d", i, pid,  
+	       WTERMSIG (status));
       }
       else
       {
@@ -294,11 +423,13 @@ void server_reset_signal(void)
     sigaction(SIGALRM, &sv, 0 );
     sigaction(SIGHUP,  &sv, 0 );
     sigaction(SIGTERM, &sv, 0 );
+    sigaction(SIGUSR1, &sv, 0 );
     sigaction(SIGCHLD, &sv, 0 );
     
     sigemptyset(&sigs);
     sigaddset(&sigs, SIGALRM);
     sigaddset(&sigs, SIGHUP);
+    sigaddset(&sigs, SIGUSR1);
     sigaddset(&sigs, SIGCHLD);
     sigprocmask(SIG_UNBLOCK, &sigs, NULL);
         
